//{$A+,B-,D+,F-,G+,I+,K+,L+,N+,P+,Q-,R-,S+,T-,V-,W-,X+,Y+}
unit UJetCheckBox;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Buttons, stdctrls, PicClip, extctrls;

type
  EGeneralError = class(Exception);
  EMouseLeave = procedure (Sender: TObject) of object;
  EMouseEnter = procedure (Sender: TObject) of object;
  EAfterCheckedChange = procedure (Sender: TObject) of object;

  TButtonType = (btRadioButton, btCheckButton);

  TJetCheckBox = class(TGraphicControl)
  private
    FAutoSize: Boolean;
    FBitmapUnchecked: TBitmap;
    FBitmapChecked: TBitmap;
    FBitmapOverUnchecked: TBitmap;
    FBitmapOverChecked: TBitmap;
    FBitmapDisabled: TBitmap;
    FButtonType: TButtonType;
    FPicClip: TPicClip;
    FPicSeries: String;
    FChecked : boolean;
    FMouseLeave: EMouseLeave;
    FMouseEnter: EMouseEnter;
    FAfterCheckedChange: EAfterCheckedChange;
    FMouseEnabled: Boolean;
    FGroupIndex: Integer;
    TempBitmap : TBitmap;
    MouseOver : Boolean;
    FTransparentColor : TColor;
    OldState: TCheckBoxState;
    FBitmapEditor: String;
    PTimer: TTimer;
    procedure AdjustBounds;
    procedure BitmapUncheckedChanged(Sender: TObject);
    procedure BitmapCheckedChanged(Sender: TObject);
    procedure BitmapOverUncheckedChanged(Sender: TObject);
    procedure BitmapOverCheckedChanged(Sender: TObject);
    procedure BitmapDisabledChanged(Sender: TObject);
    procedure SetBitmapChecked(Value: TBitmap);
    procedure SetBitmapUnchecked(Value: TBitmap);
    procedure SetBitmapOverUnchecked(Value: TBitmap);
    procedure SetBitmapOverChecked(Value: TBitmap);
    procedure SetBitmapDisabled(Value: TBitmap);
    procedure SetPicClip(Value: TPicClip);
    procedure SetPicSeries(Value: String);
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMSysColorChange(var Message: TMessage); message CM_SYSCOLORCHANGE;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
//    procedure CMChangGroup(var Message: TMessage); message CM_CHANGEGROUP;
    procedure SetChecked(Value : boolean);
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    procedure Invalidate; override;
    function PtInMask(const X, Y: Integer): Boolean;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure SetEnabled(Value : Boolean);
    procedure PTimerTime(Sender: TObject);
    function ExtractFieldName(const Fields: string; var Pos: Integer): string;
  protected
    FState: TCheckBoxState;
    FEnabled : boolean;
    procedure Toggle;
    function GetPalette: HPALETTE; override;
    procedure DrawButtonText(Canvas: TCanvas; const Caption: String; TextBounds: TRect; State: TCheckBoxState);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;  X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;  X, Y: Integer); override;
    procedure Paint; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    procedure CopyBitmaps(JetCheckBox: TJetCheckBox);
  published
    property Checked : Boolean read FChecked write SetChecked;
    property TransparentColor : TColor read FTransparentColor write FTransparentColor;
    property BitmapUnchecked: TBitmap read FBitmapUnchecked write SetBitmapUnchecked;
    property BitmapChecked: TBitmap read FBitmapChecked write SetBitmapChecked;
    property BitmapOverUnchecked: TBitmap read FBitmapOverUnchecked write SetBitmapOverUnchecked;
    property BitmapOverChecked: TBitmap read FBitmapOverChecked write SetBitmapOverChecked;
    property BitmapDisabled: TBitmap read FBitmapDisabled write SetBitmapDisabled;
    property BitmapEditor: String read FBitmapEditor write FBitmapEditor;
    property PicClip: TPicClip read FPicClip write SetPicClip;
    property PicSeries: String read FPicSeries write SetPicSeries;
    property ButtonType: TButtonType read FButtonType write FButtonType;
    property GroupIndex: Integer read FGroupIndex write FGroupIndex;
    property Caption;
    property Enabled : Boolean read FEnabled write SetEnabled;
    property MouseEnabled: Boolean read FMouseEnabled write FMouseEnabled;
    property Font;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseLeave: EMouseLeave read FMouseLeave write FMouseLeave;
    property OnMouseEnter: EMouseEnter read FMouseEnter write FMouseEnter;
    property AfterCheckedChange: EAfterCheckedChange read FAfterCheckedChange write FAfterCheckedChange;
  end;

implementation

function MakeMask(const ColorBmp: TBitmap; TransparentColor: TColor): TBitmap;
var Temp: TRect;
    OldBkColor: TColorRef;
    TmpBitmap : Tbitmap;
begin
  Makemask := nil;
  TmpBitmap := TBitmap.Create;
  try
    TmpBitmap.Monochrome := True;
    TmpBitmap.Width := ColorBmp.Width;
    TmpBitmap.Height := ColorBmp.Height;
    OldBkColor := SetBkColor(ColorBmp.Canvas.Handle, ColorToRGB(TransparentColor));
    Temp := Rect(0, 0, ColorBmp.Width, ColorBmp.Height);
    TmpBitmap.Canvas.CopyMode := cmSrcCopy;
    TmpBitmap.Canvas.CopyRect(Temp, ColorBmp.Canvas, Temp);
    SetBkColor(ColorBmp.Canvas.Handle, OldBkColor);
    MakeMask := TmpBitmap;
  except
    TmpBitmap.Free;
  end;
end;

constructor TJetCheckBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetBounds(0, 0, 50, 50);
  ControlStyle := [csCaptureMouse, csOpaque];
  FAutoSize := True;
  FBitmapUnchecked := TBitmap.Create;
  FBitmapUnchecked.OnChange := BitmapUncheckedChanged;
  FBitmapChecked := TBitmap.Create;
  FBitmapChecked.OnChange := BitmapCheckedChanged;
  FBitmapOverUnchecked := TBitmap.Create;
  FBitmapOverUnchecked.OnChange := BitmapOverUncheckedChanged;
  FBitmapOverChecked := TBitmap.Create;
  FBitmapOverChecked.OnChange := BitmapOverCheckedChanged;
  FBitmapDisabled := TBitmap.Create;
  FBitmapDisabled.OnChange := BitmapDisabledChanged;
  FTransparentColor := clWhite;
  TempBitmap := nil;
  ParentFont := True;
  FEnabled := true;
  FChecked := false;
  MouseOver := false;
  FState := cbUnchecked;
  FMouseEnabled:=True;
  PTimer:=TTimer.Create(Self);
  PTimer.Enabled:=False;
  PTimer.Interval:=100;
  PTimer.OnTimer:=PTimerTime;
  FButtonType:=btCheckButton;
end;

destructor TJetCheckBox.Destroy;
begin
  FBitmapUnchecked.Free;
  FBitmapChecked.Free;
  FBitmapOverUnchecked.Free;
  FBitmapOverChecked.Free;
  FBitmapDisabled.Free;
  TempBitmap.Free;
  FPicClip:=nil;
  PTimer.Free;
  inherited Destroy;
end;

procedure TJetCheckBox.Paint;
var CurrentBmp: TBitmap;
    R: TRect;
begin
  if csDesigning in ComponentState then begin
    with Canvas do begin
      Pen.Style := psSolid;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;
  end;
  if (FState in [cbGrayed]) then
  begin
     if Not FBitmapDisabled.Empty then CurrentBmp := FBitmapDisabled else
     CurrentBmp := FBitmapUnchecked;
  end else if (FState = cbUnchecked) and (not MouseOver) then CurrentBmp := FBitmapUnchecked
  else if (FState = cbUnchecked) and MouseOver then
  begin
    if Not FBitmapOverUnchecked.Empty then CurrentBmp := FBitmapOverUnchecked else CurrentBmp := FBitmapUnchecked;
  end else
  if (FState = cbChecked) and (not MouseOver) then CurrentBmp := FBitmapChecked else
  begin
    if not FBitmapOverChecked.empty then CurrentBmp := FBitmapOverChecked else CurrentBmp := FBitmapChecked;
  end;

  With CurrentBmp do begin
    Transparent := true;
    TransparentMode := tmAuto;
  end;
  Canvas.Draw(0,0,CurrentBmp);

  if Length(Caption) > 0 then
  begin
    Canvas.Font := Self.Font;
    R := CLIENTRECT;
    DrawButtonText(Canvas, Caption, R, FState);
  end;
end;

function TJetCheckBox.PtInMask(const X, Y: Integer): Boolean;
begin
  Result := True;
  if TempBitmap <> nil then
    Result := (TempBitmap.Canvas.Pixels[X, Y] = clBlack);
end;

procedure TJetCheckBox.Toggle;
begin
  Case FState of
   cbUnchecked : FState := cbChecked;
   cbChecked : FState := cbUnchecked;
//   cbGrayed :;
  end;
end;

procedure TJetCheckBox.MouseMove(Shift: TShiftState; X, Y: Integer);
var Last : Boolean;
begin
  inherited MouseMove(Shift, X, Y);
  if not ((Enabled) and (FMouseEnabled) and ((FBitmapOverUnchecked.Empty=False) or (FBitmapOverChecked.Empty=False))) then Exit;
  Last := MouseOver;
  MouseOver := PtInMask(X, Y);
  PTimer.Enabled:=MouseOver;
  if Last <> MouseOver then
  begin
    IF (MouseOver) and (Assigned(FMouseEnter)) and (Enabled) then FMouseEnter(Self);
    IF (MouseOver=False) and (Assigned(FMouseLeave)) and (Enabled) then FMouseLeave(Self);
    if FBitmapUnchecked.Empty and Enabled then Invalidate
    else Repaint;
  end;
end;

procedure TJetCheckBox.PTimerTime(Sender: TObject);
Var P:TPoint;
begin
  GetCursorPos(P);
  P:=ScreenToClient(P);
  If (PtInMask(P.X,P.Y)=False) then begin
    PTimer.Enabled:=False;
    IF (MouseOver) and (Assigned(FMouseLeave)) and (Enabled) then FMouseLeave(Self);
    MouseOver := false;
    PTimer.Enabled:=MouseOver;
    if FBitmapUnchecked.Empty and Enabled then Invalidate
    else Repaint;
  end;
end;

procedure TJetCheckBox.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var Clicked: Boolean;
begin
  If FMouseEnabled=False then Exit;
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and Enabled then
  begin
    Clicked := PtInMask(X, Y);
    if Clicked then
    begin
      MouseOver := true;
      PTimer.Enabled:=MouseOver;
      Repaint;
    end;
  end;
end;

procedure TJetCheckBox.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
  If FMouseEnabled=False then Exit;
  inherited MouseUp(Button, Shift, X, Y);
  If (Button<>mbLeft) or (Enabled=False) then Exit;
  DoClick := PtInMask(X, Y);
  if DoClick then
  begin
    MouseOver := true;
    PTimer.Enabled:=MouseOver;
    Toggle;
    Click;
    Repaint;
  end;
end;

procedure TJetCheckBox.Click;
Var X: Integer;
begin
  IF (Enabled) and (FMouseEnabled) then begin
    Checked:=not Checked;
    If (Checked) and (GroupIndex>0) then begin
      For X:=0 to Parent.ControlCount-1 do begin
        If (Parent.Controls[X] is TJetCheckBox) and
           ((Parent.Controls[X] as TJetCheckBox) <> Self) and
           ((Parent.Controls[X] as TJetCheckBox).GroupIndex=GroupIndex) and
           ((Parent.Controls[X] as TJetCheckBox).Checked) then begin
          (Parent.Controls[X] as TJetCheckBox).Checked:=False;
        end;
      end;
    end;
    inherited Click;
  end;
end;

function TJetCheckBox.GetPalette: HPALETTE;
begin
  Result := FBitmapUnchecked.Palette;
end;

procedure TJetCheckBox.SetBitmapUnchecked(Value: TBitmap);
begin
  FBitmapUnchecked.Assign(Value);
end;

procedure TJetCheckBox.SetBitmapChecked(Value: TBitmap);
begin
  FBitmapChecked.Assign(Value);
end;

procedure TJetCheckBox.SetBitmapOverUnchecked(Value: TBitmap);
begin
  FBitmapOverUnchecked.Assign(Value);
end;

procedure TJetCheckBox.SetBitmapOverChecked(Value: TBitmap);
begin
  FBitmapOverChecked.Assign(Value);
end;

procedure TJetCheckBox.SetBitmapDisabled(Value: TBitmap);
begin
  FBitmapDisabled.Assign(Value);
end;

procedure TJetCheckBox.SetPicClip(Value: TPicClip);
begin
  If Value<>FPicClip then begin
    If Value<>nil then begin
      Value.FreeNotification(Self);
//      If FPicSeries='' then begin
        Case Value.Cols of
          1: FPicSeries:='Unchecked';
          2: FPicSeries:='Unchecked;Checked';
          3: FPicSeries:='Unchecked;Checked;Disabled';
          4: FPicSeries:='Unchecked;Disabled;Checked;OverUnchecked';
          5: FPicSeries:='Unchecked;Disabled;Checked;OverUnchecked;OverChecked';
        end;
//      end;
    end;
    FPicClip:=Value;
  end;
end;

procedure TJetCheckBox.SetPicSeries(Value: String);
Var X: Integer;
    S: String;
begin
  If Value<>FPicSeries then begin
    Value:=Trim(Value);
    If Value<>'' then begin
      X:=1;
      while X<=Length(Value) do begin
        S:=AnsiUpperCase(ExtractFieldName(Value,X));
        If not ((S='UNCHECKED') or (S='DISABLED') or (S='CHECKED') or (S='OVERUNCHECKED') or (S='OVERCHECKED')) then
          raise EGeneralError.Create('"'+S+'" is not a valid PicSeries value. Available values: Unchecked;Disabled;Checked;OverUnchecked;OverChecked');
      end;
      FPicSeries:=Value;
    end
    else FPicSeries:='';
  end;
end;

procedure TJetCheckBox.BitmapUncheckedChanged(Sender: TObject);
Var Maskbmp : TBitmap;
    R : TRect;
begin
  AdjustBounds;
  MaskBmp := TBitmap.create;
  MaskBmp.Width := FBitmapUnchecked.width;
  MaskBmp.Height := FBitmapUnchecked.Height;
  R := Rect(0,0,FBitmapUnchecked.Width,FBitmapUnchecked.Height);
  MaskBmp.Canvas.CopyRect(R, FBitmapUnchecked.Canvas, R);
  FTransparentColor := FBitmapUnchecked.TransparentColor;
  TempBitmap.Free;
  TempBitmap := MakeMask(MaskBmp, FTransparentColor);
  MaskBmp.free;
  Invalidate;
end;

procedure TJetCheckBox.BitmapCheckedChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TJetCheckBox.BitmapOverCheckedChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TJetCheckBox.BitmapOverUncheckedChanged(Sender: TObject);
//Var R : TRect;
begin
{  if not FBitmapUnchecked.Empty then
  begin
    SetBounds(Left, Top, FBitmapOverUnchecked.Width, FBitmapOverUnchecked.Height);
    MaskBmp := TBitmap.create;
    MaskBmp.Width := FBitmapOverUnchecked.width;
    MaskBmp.Height := FBitmapOverUnchecked.Height;
    R := Rect(0,0,FBitmapOverUnchecked.Width,FBitmapOverUnchecked.Height);
    FTransparentColor := FBitmapOverUnchecked.TransparentColor;
    MaskBmp.Canvas.CopyRect(R, FBitmapOverUnchecked.Canvas, R);
    TempBitmap.Free;
    TempBitmap := MakeMask(MaskBmp, FTransparentColor);
    MaskBmp.free;
  end;}
  Invalidate;
end;

procedure TJetCheckBox.BitmapDisabledChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TJetCheckBox.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure TJetCheckBox.CMFontChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TJetCheckBox.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TJetCheckBox.CMSysColorChange(var Message: TMessage);
begin
  BitmapUncheckedChanged(Self);
  BitmapCheckedChanged(Self);
  BitmapOverUncheckedChanged(Self);
  BitmapOverCheckedChanged(Self);
  BitmapDisabledChanged(Self);
end;

procedure TJetCheckBox.CMMouseEnter(var Message: TMessage);
begin
  inherited;
end;

procedure TJetCheckBox.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  IF MouseOver then begin
    IF (Assigned(FMouseLeave)) and (Enabled) then FMouseLeave(Self);
    MouseOver := false;
    PTimer.Enabled:=MouseOver;
    if FBitmapUnchecked.Empty and Enabled then Invalidate else Repaint;
  end;
end;

{procedure TJetCheckBox.CMChangGroup(var Message: TMessage);
begin
end;}

procedure TJetCheckBox.DrawButtonText(Canvas: TCanvas; const Caption: String;
  TextBounds: TRect; State: TCheckBoxState);
var
  CString: array[0..255] of Char;
begin
  StrPCopy(CString, Caption);
  Canvas.Brush.Style := bsClear;
  if State = cbChecked then OffsetRect(TextBounds, 1, 1);
  DrawText(Canvas.Handle, CString, -1, TextBounds,
      DT_CENTER or DT_VCENTER or DT_SINGLELINE);
end;

procedure TJetCheckBox.Loaded;
Var X,Count: Integer;
    S: String;
begin
  inherited Loaded;
  if (FBitmapUnchecked <> nil) and (FBitmapUnchecked.Width > 0) and (FBitmapUnchecked.Height > 0) then
  begin
    BitmapUncheckedChanged(Self);
  end else if (FBitmapOverUnchecked <> nil) and (FBitmapOverUnchecked.Width > 0) and (FBitmapOverUnchecked.Height > 0) then
  begin
    BitmapOverUncheckedChanged(Self);
  end;
  If (FPicSeries<>'') and (Assigned(FPicClip)) and
     (FPicClip.Picture<>nil) and (not (csDesigning in ComponentState)) then begin
    X:=1;
    Count:=1;
    while X<=Length(FPicSeries) do begin
      S:=AnsiUpperCase(ExtractFieldName(FPicSeries,X));
      If Count<=FPicClip.Cols then begin
        If (S='UNCHECKED') then FBitmapUnchecked.Assign(FPicClip.GraphicCell[Count-1]);
        If (S='DISABLED') then FBitmapDisabled.Assign(FPicClip.GraphicCell[Count-1]);
        If (S='CHECKED') then FBitmapChecked.Assign(FPicClip.GraphicCell[Count-1]);
        If (S='OVERUNCHECKED') then FBitmapOverUnchecked.Assign(FPicClip.GraphicCell[Count-1]);
        If (S='OVERCHECKED') then FBitmapOverChecked.Assign(FPicClip.GraphicCell[Count-1]);
      end;
      Inc(Count);
    end;
    Invalidate;
  end;
  If (FChecked) and (FButtonType = btRadioButton) then FMouseEnabled:=False;

{  With FBitmapUnchecked do begin
    Transparent := true;
    TransparentMode := tmAuto;
  end;
  With FBitmapDisabled do begin
    Transparent := true;
    TransparentMode := tmAuto;
  end;
  With FBitmapChecked do begin
    Transparent := true;
    TransparentMode := tmAuto;
  end;
  With FBitmapOverUnchecked do begin
    Transparent := true;
    TransparentMode := tmAuto;
  end;
  With FBitmapOverChecked do begin
    Transparent := true;
    TransparentMode := tmAuto;
  end;}
end;

procedure TJetCheckBox.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation = opRemove) and (Assigned(AComponent)) and (AComponent=FPicClip) then PicClip:=nil;
end;

procedure TJetCheckBox.CopyBitmaps(JetCheckBox: TJetCheckBox);
begin
  BitmapUnchecked:=JetCheckBox.BitmapUnchecked;
  BitmapOverUnchecked:=JetCheckBox.BitmapOverUnchecked;
  BitmapChecked:=JetCheckBox.BitmapChecked;
  BitmapOverChecked:=JetCheckBox.BitmapOverChecked;
  BitmapDisabled:=JetCheckBox.BitmapDisabled;
end;

procedure TJetCheckBox.AdjustBounds;
begin
  SetBounds(Left, Top, Width, Height);
end;

procedure TJetCheckBox.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var W, H: Integer;
begin
  W := AWidth;
  H := AHeight;
  if not (csReading in ComponentState) and FAutoSize and not FBitmapUnchecked.Empty then
  begin
    W := FBitmapUnchecked.Width;
    H := FBitmapUnchecked.Height;
  end;
{  If (W=0) or (H=0) then begin
    W := AWidth;
    H := AHeight;
  end;}
  inherited SetBounds(ALeft, ATop, W, H);
end;

procedure TJetCheckBox.Invalidate;
var R: TRect;
begin
  if (Visible or (csDesigning in ComponentState)) and
    (Parent <> nil) and Parent.HandleAllocated then
  begin
    R := BoundsRect;
    InvalidateRect(Parent.Handle, @R, True);
  end;
end;

procedure TJetCheckBox.SetEnabled(Value : Boolean);
begin
 if Value <> FEnabled then
 begin
   FEnabled := Value;
   if FEnabled = false then begin
     OldState:=FState;
     FState := cbGrayed
   end
   else FState := OldState;
   Invalidate;
 end;
end;

procedure TJetCheckBox.SetChecked(Value : boolean);
Var X: Integer;
begin
  if (Value <> FChecked) and (Enabled) then
  begin
    FChecked := Value;
    if FChecked = true then FState := cbChecked else FState := cbUnchecked;

    If (FChecked) and (GroupIndex>0) then begin
      For X:=0 to Parent.ControlCount-1 do begin
        If (Parent.Controls[X] is TJetCheckBox) and
           ((Parent.Controls[X] as TJetCheckBox) <> Self) and
           ((Parent.Controls[X] as TJetCheckBox).GroupIndex=GroupIndex) and
           ((Parent.Controls[X] as TJetCheckBox).FChecked) then begin

          (Parent.Controls[X] as TJetCheckBox).Checked:=False;
          If (Parent.Controls[X] as TJetCheckBox).ButtonType = btRadioButton then
            (Parent.Controls[X] as TJetCheckBox).MouseEnabled:=True;
        end;
      end;
      If FButtonType = btRadioButton then FMouseEnabled:=False;
    end;
    If Assigned(FAfterCheckedChange) then FAfterCheckedChange(Self);
    Invalidate;
  end;
end;

function TJetCheckBox.ExtractFieldName(const Fields: string; var Pos: Integer): string;
var
  I: Integer;
begin
  I := Pos;
  while (I <= Length(Fields)) and (Fields[I] <> ';') do Inc(I);
  Result := Trim(Copy(Fields, Pos, I - Pos));
  if (I <= Length(Fields)) and (Fields[I] = ';') then Inc(I);
  Pos := I;
end;

end.
