//{$A+,B-,D+,F-,G+,I+,K+,L+,N+,P+,Q-,R-,S+,T-,V-,W-,X+,Y+}
unit UjetButton;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Buttons, PicClip, extctrls;

type
  EGeneralError = class(Exception);
  EMouseLeave = procedure (Sender: TObject) of object;
  EMouseEnter = procedure (Sender: TObject) of object;

  TJetButton = class(TGraphicControl)
  private
    FAutoSize: Boolean;
    FPicClip: TPicClip;
    FPicSeries: String;
    FBitmapUp: TBitmap;
    FBitmapDown: TBitmap;
    FBitmapOver: TBitmap;
    FBitmapDisabled: TBitmap;
    TempBitmap : TBitmap;
    MouseOver : Boolean;
    FTransparentColor : TColor;
    FBitmapEditor: String;
    PTimer: TTimer;
    FMouseLeave: EMouseLeave;
    FMouseEnter: EMouseEnter;
    procedure AdjustBounds;
    procedure BitmapUpChanged(Sender: TObject);
    procedure BitmapDownChanged(Sender: TObject);
    procedure BitmapOverChanged(Sender: TObject);
    procedure BitmapDisabledChanged(Sender: TObject);
    procedure SetBitmapDown(Value: TBitmap);
    procedure SetBitmapUp(Value: TBitmap);
    procedure SetBitmapOver(Value: TBitmap);
    procedure SetBitmapDisabled(Value: TBitmap);
    procedure SetPicClip(Value: TPicClip);
    procedure SetPicSeries(Value: String);
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMSysColorChange(var Message: TMessage); message CM_SYSCOLORCHANGE;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    procedure Invalidate; override;
    function PtInMask(const X, Y: Integer): Boolean;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure PTimerTime(Sender: TObject);
    function ExtractFieldName(const Fields: string; var Pos: Integer): string;
  protected
    FState: TButtonState;
    FEnabled : boolean;
    function GetPalette: HPALETTE; override;
    procedure SetEnabled(Value : Boolean);
    procedure DrawButtonText(Canvas: TCanvas; const Caption: String; TextBounds: TRect; State: TButtonState);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;  X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;  X, Y: Integer); override;
    procedure Paint; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    procedure CopyBitmaps(JetButton: TJetButton);
  published
    property TransparentColor : TColor read FTransparentColor write FTransparentColor;
    property BitmapUp: TBitmap read FBitmapUp write SetBitmapUp;
    property BitmapDown: TBitmap read FBitmapDown write SetBitmapDown;
    property BitmapOver: TBitmap read FBitmapOver write SetBitmapOver;
    property BitmapDisabled: TBitmap read FBitmapDisabled write SetBitmapDisabled;
    property BitmapEditor: String read FBitmapEditor write FBitmapEditor;
    property PicClip: TPicClip read FPicClip write SetPicClip;
    property PicSeries: String read FPicSeries write SetPicSeries;
    property Caption;
    property Enabled : Boolean read FEnabled write SetEnabled;
    property Font;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseLeave: EMouseLeave read FMouseLeave write FMouseLeave;
    property OnMouseEnter: EMouseEnter read FMouseEnter write FMouseEnter;
  end;

implementation

function MakeMask(const ColorBmp: TBitmap; TransparentColor: TColor): TBitmap;
var Temp: TRect;
    OldBkColor: TColorRef;
    TmpBitmap : Tbitmap;
begin
  Makemask := nil;
  TmpBitmap := TBitmap.Create;
  try
    TmpBitmap.Monochrome := True;
    TmpBitmap.Width := ColorBmp.Width;
    TmpBitmap.Height := ColorBmp.Height;
    OldBkColor := SetBkColor(ColorBmp.Canvas.Handle, ColorToRGB(TransparentColor));
    Temp := Rect(0, 0, ColorBmp.Width, ColorBmp.Height);
    TmpBitmap.Canvas.CopyMode := cmSrcCopy;
    TmpBitmap.Canvas.CopyRect(Temp, ColorBmp.Canvas, Temp);
    SetBkColor(ColorBmp.Canvas.Handle, OldBkColor);
    MakeMask := TmpBitmap;
  except
    TmpBitmap.Free;
  end;
end;

constructor TJetButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetBounds(0, 0, 50, 50);
  ControlStyle := [csCaptureMouse, csOpaque];
  FAutoSize := True;
  FBitmapUp := TBitmap.Create;
  FBitmapUp.OnChange := BitmapUpChanged;
  FBitmapDown := TBitmap.Create;
  FBitmapDown.OnChange := BitmapDownChanged;
  FBitmapOver := TBitmap.Create;
  FBitmapOver.OnChange := BitmapOverChanged;
  FBitmapDisabled := TBitmap.Create;
  FBitmapDisabled.OnChange := BitmapDisabledChanged;
  FTransparentColor := clWhite;
  TempBitmap := nil;
  ParentFont := True;
  FEnabled := true;
  MouseOver := false;
  FState := bsUp;
  PTimer:=TTimer.Create(Self);
  PTimer.Enabled:=False;
  PTimer.Interval:=100;
  PTimer.OnTimer:=PTimerTime;
end;

destructor TJetButton.Destroy;
begin
  FBitmapUp.Free;
  FBitmapDown.Free;
  FBitmapOver.Free;
  FBitmapDisabled.Free;
  TempBitmap.Free;
  FPicClip:=nil;
  PTimer.Free;
  inherited Destroy;
end;

procedure TJetButton.Paint;
var CurrentBmp: TBitmap;
    R: TRect;
begin
  if csDesigning in ComponentState then
    with Canvas do
    begin
      Pen.Style := psSolid;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;

  if (FState in [bsDisabled, bsExclusive]) then
  begin
     if Not FBitmapDisabled.Empty then CurrentBmp := FBitmapDisabled else CurrentBmp := FBitmapUp;
  end else
  if (FState = bsUp) and (not MouseOver) then CurrentBmp := FBitmapUp
  else
  if (FState = bsUp) and MouseOver then
  begin
    if Not FBitmapOver.Empty then CurrentBmp := FBitmapOver else CurrentBmp := FBitmapUp;
  end else
  begin
    if Not FBitmapDown.Empty then CurrentBmp := FBitmapDown else CurrentBmp := FBitmapUp;
  end;

  With CurrentBmp do begin
    Transparent := true;
    TransparentMode := tmAuto;
  end;
  Canvas.Draw(0,0,CurrentBmp);

  if Length(Caption) > 0 then
  begin
    Canvas.Font := Self.Font;
    R := CLIENTRECT;
    DrawButtonText(Canvas, Caption, R, FState);
  end;

end;

function TJetButton.PtInMask(const X, Y: Integer): Boolean;
begin
  Result := True;
  if TempBitmap <> nil then
    Result := (TempBitmap.Canvas.Pixels[X, Y] = clBlack);
end;

procedure TJetButton.MouseMove(Shift: TShiftState; X, Y: Integer);
var Last : Boolean;
begin
  inherited MouseMove(Shift, X, Y);
  if (Enabled=False) or (FBitmapOver.Empty) then Exit;
  Last := MouseOver;
  MouseOver := PtInMask(X, Y);
  PTimer.Enabled:=MouseOver;
  if (Last <> MouseOver) and (not (FState = bsDown)) then
  begin
    IF (MouseOver) and (Assigned(FMouseEnter)) and (Enabled) then FMouseEnter(Self);
    IF (MouseOver=False) and (Assigned(FMouseLeave)) and (Enabled) then FMouseLeave(Self);
    if FBitmapUp.Empty and Enabled then Invalidate
    else Repaint;
  end;
end;

procedure TJetButton.PTimerTime(Sender: TObject);
Var P:TPoint;
begin
  GetCursorPos(P);
  P:=ScreenToClient(P);
  If PtInMask(P.X,P.Y)=False then begin
    PTimer.Enabled:=False;
    IF (MouseOver) and (Assigned(FMouseLeave)) and (Enabled) then FMouseLeave(Self);
    MouseOver := false;
    if FBitmapUp.Empty and Enabled then Invalidate
    else Repaint;
  end;
end;

procedure TJetButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var Clicked: Boolean;
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and Enabled then
  begin
    Clicked := PtInMask(X, Y);
    if Clicked then
    begin
      FState := bsDown;
//      MouseOver := true;
      Repaint;
    end;
  end;
end;

procedure TJetButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
  inherited MouseUp(Button, Shift, X, Y);
  If (Button<>mbLeft) or (Enabled=False) then Exit;
  DoClick := PtInMask(X, Y);
  If (FState = bsDown) then begin
    FState := bsUp;
    Repaint;
    if FBitmapUp.Empty and Enabled then Invalidate else Repaint;
  end;
  if DoClick then begin
    MouseOver := true;
    PTimer.Enabled:=MouseOver;
    Click;
  end;
end;

procedure TJetButton.Click;
begin
  inherited Click;
end;

function TJetButton.GetPalette: HPALETTE;
begin
  Result := FBitmapUp.Palette;
end;

procedure TJetButton.SetBitmapUp(Value: TBitmap);
begin
  FBitmapUp.Assign(Value);
end;

procedure TJetButton.SetBitmapDown(Value: TBitmap);
begin
  FBitmapDown.Assign(Value);
end;

procedure TJetButton.SetBitmapOver(Value: TBitmap);
begin
  FBitmapOver.Assign(Value);
end;

procedure TJetButton.SetBitmapDisabled(Value: TBitmap);
begin
  FBitmapDisabled.Assign(Value);
end;

procedure TJetButton.SetPicClip(Value: TPicClip);
begin
  If Value<>FPicClip then begin
    If Value<>nil then begin
      Value.FreeNotification(Self);
//      If FPicSeries='' then begin
        Case Value.Cols of
          1: FPicSeries:='Up';
          2: FPicSeries:='Up;Down';
          3: FPicSeries:='Up;Down;Disabled';
          4: FPicSeries:='Up;Disabled;Down;Over';
        end;
//      end;
    end;
    FPicClip:=Value;
  end;
end;

procedure TJetButton.SetPicSeries(Value: String);
Var X: Integer;
    S: String;
begin
  If Value<>FPicSeries then begin
    Value:=Trim(Value);
    If Value<>'' then begin
      X:=1;
      while X<=Length(Value) do begin
        S:=AnsiUpperCase(ExtractFieldName(Value,X));
        If not ((S='UP') or (S='DOWN') or (S='DISABLED') or (S='OVER')) then
          raise EGeneralError.Create('"'+S+'" is not a valid PicSeries value. Available values: Up;Down;Disabled;Over');
      end;
      FPicSeries:=Value;
    end
    else FPicSeries:='';
  end;
end;

procedure TJetButton.BitmapUpChanged(Sender: TObject);
Var Maskbmp : TBitmap;
    R : TRect;
begin
  AdjustBounds;
  MaskBmp := TBitmap.create;
  MaskBmp.Width := FBitmapUp.width;
  MaskBmp.Height := FBitmapUp.Height;
  R := Rect(0,0,FBitmapUp.Width,FBitmapUp.Height);
  MaskBmp.Canvas.CopyRect(R, FBitmapUp.Canvas, R);
  FTransparentColor := FBitmapUp.TransparentColor;
  TempBitmap.Free;
  TempBitmap := MakeMask(MaskBmp, FTransparentColor);
  MaskBmp.free;
  Invalidate;
end;

procedure TJetButton.BitmapDownChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TJetButton.BitmapOverChanged(Sender: TObject);
{Var Maskbmp : TBitmap;
    R : TRect;}
begin
{  if not FBitmapUp.Empty then
  begin
    SetBounds(Left, Top, FBitmapOver.Width, FBitmapOver.Height);
    MaskBmp := TBitmap.create;
    MaskBmp.Width := FBitmapOver.width;
    MaskBmp.Height := FBitmapOver.Height;
    R := Rect(0,0,FBitmapOver.Width,FBitmapOver.Height);
    FTransparentColor := FBitmapOver.TransparentColor;
    MaskBmp.Canvas.CopyRect(R, FBitmapOver.Canvas, R);
    TempBitmap.Free;
    TempBitmap := MakeMask(MaskBmp, FTransparentColor);
    MaskBmp.free;
  end;}
  Invalidate;
end;

procedure TJetButton.BitmapDisabledChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TJetButton.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure TJetButton.CMFontChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TJetButton.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TJetButton.CMSysColorChange(var Message: TMessage);
begin
  BitmapUpChanged(Self);
  BitmapDownChanged(Self);
  BitmapOverChanged(Self);
  BitmapDisabledChanged(Self);
end;

procedure TJetButton.CMMouseEnter(var Message: TMessage);
begin
  inherited;
end;

procedure TJetButton.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  IF (MouseOver) and (Assigned(FMouseLeave)) and (Enabled) then FMouseLeave(Self);
  MouseOver := false;
  PTimer.Enabled:=MouseOver;
  if (Fstate = bsDown) then exit;
  if FBitmapUp.Empty and Enabled then Invalidate else Repaint;
end;

procedure TJetButton.DrawButtonText(Canvas: TCanvas; const Caption: String;
  TextBounds: TRect; State: TButtonState);
var
  CString: array[0..255] of Char;
begin
  StrPCopy(CString, Caption);
  Canvas.Brush.Style := bsClear;
  if State = bsDown then OffsetRect(TextBounds, 1, 1);
  DrawText(Canvas.Handle, CString, -1, TextBounds,
      DT_CENTER or DT_VCENTER or DT_SINGLELINE);
end;

procedure TJetButton.Loaded;
Var X,Count: Integer;
    S: String;
begin
  inherited Loaded;
  if (FBitmapUp <> nil) and (FBitmapUp.Width > 0) and (FBitmapUp.Height > 0) then
  begin
    BitmapUpChanged(Self);
  end else if (FBitmapOver <> nil) and (FBitmapOver.Width > 0) and (FBitmapOver.Height > 0) then
  begin
    BitmapOverChanged(Self);
  end;
  If (FPicSeries<>'') and (Assigned(FPicClip)) and
     (FPicClip.Picture<>nil) and (not (csDesigning in ComponentState)) then begin
    X:=1;
    Count:=1;
    while X<=Length(FPicSeries) do begin
      S:=AnsiUpperCase(ExtractFieldName(FPicSeries,X));
      If Count<=FPicClip.Cols then begin
        If (S='UP') then FBitmapUp.Assign(FPicClip.GraphicCell[Count-1]);
        If (S='DISABLED') then FBitmapDisabled.Assign(FPicClip.GraphicCell[Count-1]);
        If (S='DOWN') then FBitmapDown.Assign(FPicClip.GraphicCell[Count-1]);
        If (S='OVER') then FBitmapOver.Assign(FPicClip.GraphicCell[Count-1]);
      end;
      Inc(Count);
    end;
    Invalidate;
  end;
end;

procedure TJetButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation = opRemove) and (Assigned(AComponent)) and (AComponent=FPicClip) then PicClip:=nil;
end;

procedure TJetButton.CopyBitmaps(JetButton: TJetButton);
begin
  BitmapUp:=JetButton.BitmapUp;
  BitmapDown:=JetButton.BitmapDown;
  BitmapOver:=JetButton.BitmapOver;
  BitmapDisabled:=JetButton.BitmapDisabled;
end;

procedure TJetButton.AdjustBounds;
begin
  SetBounds(Left, Top, Width, Height);
end;

procedure TJetButton.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var W, H: Integer;
begin
  W := AWidth;
  H := AHeight;
  if not (csReading in ComponentState) and FAutoSize and not FBitmapUp.Empty then
  begin
    W := FBitmapUp.Width;
    H := FBitmapUp.Height;
  end;
  inherited SetBounds(ALeft, ATop, W, H);
end;

procedure TJetButton.Invalidate;
var R: TRect;
begin
  if (Visible or (csDesigning in ComponentState)) and
    (Parent <> nil) and Parent.HandleAllocated then
  begin
    R := BoundsRect;
    InvalidateRect(Parent.Handle, @R, True);
  end;
end;

procedure TJetButton.SetEnabled(Value : Boolean);
begin
 if Value <> FEnabled then
 begin
   FEnabled := Value;
   if FEnabled = false then FState := bsDisabled else FState := bsUp;
   Invalidate;
 end;
end;

function TJetButton.ExtractFieldName(const Fields: string; var Pos: Integer): string;
var
  I: Integer;
begin
  I := Pos;
  while (I <= Length(Fields)) and (Fields[I] <> ';') do Inc(I);
  Result := Trim(Copy(Fields, Pos, I - Pos));
  if (I <= Length(Fields)) and (Fields[I] = ';') then Inc(I);
  Pos := I;
end;


end.
